#ABC-acoustic version 1.0
#S1 File: R-script for ABC analysis with simulated data
#Script 1
#For updates/upgrades of this script, please visit S.J. Puechmaille's Website; currently at http://batlab.ucd.ie/~spuechmaille/
#For a user manual of this script, please, visit S.J. Puechmaille's Website; currently at http://batlab.ucd.ie/~spuechmaille/
#Written by S.J. Puechmaille, 2017
#This script is to test the ABC and exclusion approaches based on simulated datasets (datasets being simulated as part of the script)
#This script was developped as part of the following work; Please, cite this work if you use this script.
#Lehnen L, Schorcht W, Karst I, Biedermann M, Kerth G, Puechmaille SJ (submitted) Using Approximate Bayesian Computation to infer sex ratios from acoustic data. PLoS ONE.

#Built and tested under
#R version 3.4.0 Patched (2017-05-25 r72742)
#Platform: x86_64-w64-mingw32/x64 (64-bit)
#Running under: Windows 10 x64 (build 16299)

#install.packages("EasyABC")
#install.packages("HDInterval")
#install.packages("reshape2")
	library("EasyABC")
	library(reshape2)
	library("HDInterval")


#  ###########################################################################################  #
#  ###########################################################################################  #
#  ###########################################################################################  #
                #### Part I				CREATES FUNCTIONS TO BE USED LATER		    			  ####
#  ###########################################################################################  #
#  ###########################################################################################  #
#  ###########################################################################################  #

#Sets the working directory
	setwd("D:/SEB/R/R-3.4.0patched/adata/Lisa/")
	rm(list=ls(all=TRUE))

#Creates a function to simulate a mix of males and females to be used by the ABC algorithm

	EchoSim<-function(x) {
	#Number of individuals (best to set this as the number of calls)
		Nindiv<-dim(dataE)[1]
	#Proportion of males
		PROPmal<-x[1]
	#Average peak frequency of males
		AVmal<-x[2]
	#Average peak frequency of females
		AVfem<-x[3]
	#Sd of peak frequency (same for males and females)
		SdFreq<-x[4]

	#Gets the number of males and females
		NmalSIM<-round(Nindiv*PROPmal,0)
		NfemSIM<-Nindiv-NmalSIM

	#Creates their peak frequency distribution
		MAL<<-rnorm(NmalSIM,AVmal,SdFreq)
		FEM<<-rnorm(NfemSIM,AVfem,SdFreq)
			dataSIM<<-as.data.frame(matrix(0,(NmalSIM+NfemSIM),2,dimnames=list(NULL,c("mean","sex"))))
			dataSIM$mean<<-c(MAL,FEM)
			dataSIM$sex<<-c(rep("M",NmalSIM),rep("F",NfemSIM))
	#Calculates the Kolmogorov-smirnov distance between the simulated and observed dataset
		ksD<-ks.test(c(MAL,FEM),c(dataE$mean),exact=FALSE) [1]	
		ksD1<-as.numeric(ksD[1])
	#Outputs the summary statistics
		#1.Median peak frequency (males and females together)
		#2.Mean peak frequency (males and females together)
		#3.Sd of peak frequency (males and females together)
		#4.Kolmogorov-smirnov distance between the Simulated and observed distribution of peak frequency
			c(median(c(MAL,FEM)),mean(c(MAL,FEM)),sd(c(MAL,FEM)),ksD1)
	}

#Creates a function to simulate datasets to test the method (can be considered as equivalent to true datasets)

	PRealSim<-function(Nmal,Nfem,Mfreq,Ffreq,SdFreq) {
#Simulates a dataset of known proportion of males/females 	(to test the algorithm)
#Number of males
	Nmal<-Nmal
#Number of females
	Nfem<-Nfem
#Standard deviation of peak frequency per sex
	SdFreq<-SdFreq
#Some calculations
	#Proportion of males
	MalProp<-Nmal*100/(Nmal+Nfem); MalProp
	MALsim<-rnorm(Nmal,Mfreq,SdFreq)
	FEMsim<-rnorm(Nfem,Ffreq,SdFreq)
	dataE<<-as.data.frame(matrix(0,(Nmal+Nfem),2));colnames(dataE)<-c("mean","sex")
	dataE$mean<<-c(MALsim,FEMsim)
	dataE$sex<<-c(rep("M",Nmal),rep("F",Nfem))
#Summary statistics
	sum_stat_obs<<-c(median(c(MALsim,FEMsim)),mean(c(MALsim,FEMsim)),sd(c(MALsim,FEMsim)),0) ; sum_stat_obs
}

#  ###########################################################################################  #
#  ###########################################################################################  #
#  ###########################################################################################  #
        #### Part II				SET PARAMETERS/PRIORS FOR THE SIMULATIONS AND ABC		  ####
#  ###########################################################################################  #
#  ###########################################################################################  #
#  ###########################################################################################  #
	

#### STEP 1 ####

#START of the parameters (for the simulated datasets equivalent to true datasets)
#But also some parameters from the ABC approach

#Prepares matrix with proportion of males and total pop size and all possible combinations of these 2 vectors
	#Vector or proportion of males considered
		Pmale <- seq(0, 1, by=0.5)
	#Vector of the number of calls
		Ntot <- seq(100,500, by=400)
	#Alpha parameter of the ABC algorithm 
		alpha_Lenormand<-seq(from=0.4,to=0.4,by=0.1)	#Equivalent to 0.4
	#pacc parameter of the ABC algorithm 
		pacc<-c(0.001) #pacc<-c(0.001,0.005,seq(from=0.01,to=0.05,by=0.01))
	#Takes all the vectors above and creates an object with all possible combinations of these vectors
		mordec<- expand.grid(Pmale = Pmale , Ntot = Ntot  ,Alpha= alpha_Lenormand,Pac=pacc)
	#Then calculates the number of males and females from the 'total number of calls/recordings' and 'proportion of males'
		mordec$Nmale<-round(mordec[,"Pmale"] * mordec[,"Ntot"],0)
		mordec$Nfemale<-mordec[,"Ntot"] - mordec$Nmale

#Number of simulated dataset (to be tested; i.e. equivalent to true datasets) that will be used (for each combinations of parameter available in 'mordec').
	Nrep<-5

#ABC parameters
#Seed number can be set
	set.seed(1258471) 
#Initial number of simulations
	nb_simul<-100

#Frequency parameters (for the simulated datasets equivalent to true datasets)
	#Males mean peak frequency
		MaleFreq<-106
	#Females mean peak frequency
		FemaleFreq<-109	
	#Standard deviation of peak frequency (considered as being the same for males and females)
		SdFreqFM<-1

#Run number (if results are exported, the folder containing them will have this name)
	Run<-"Run-Test1"


#### STEP 2 ####

#List with priors for the ABC algorithm
#Below, 4 priors (type of distribution, mimimum , maximum) are provided (in order:
#1-proportion of males: X1
#2-average peak frequency of males: X2
#3-average peak frequency of females: X3
#4-sd of peak frequency for males (also here sd of peak frequency for females): X4
	Rh_prior=list(c("unif",0,1),c("unif",104,107),c("unif",108,111),c("unif",0.8,1.2))

#Constraints between model parameters. Each parameter is designated with "X1", "X2", ... in the same order as in the prior definition.
	Rh_Constraints<-"X2 < X3 & X3-X2 < 5 & X3-X2 > 2.5"


#### STEP 3 ####

#Creates a large array to place all the results (and simulation parameters)
#Results are as follows;
#For each parameters (pM: proportion of males, fM: average peak frequency of males, fF: average peak frequency of females, SD: standard deviation of peak frequency)
	#the mean (mea), median (Med), lower (hl) and upper (hh) bound of the 95% highest density interval of the estimate are presented
	#e.g. pMmea is the mean proportion of males as estimated by the ABC approach. 
	#e.g. fFhh is the upper bound of the 95% highest density interval of the estimate for the average peak frequency for females
#For each of the exclusion method thresholds (95, 99 and 99.9%CI), 5 columns are presented	
	#Value of the peak frequency threshold above which 95% of females echolocate at (e.g. sFl95)
	#Number of females that have a peak frequency higher than the value of the peak frequency threshold above which 95% of females echolocate at (e.g. nF95)
	#Value of the peak frequency threshold under which 95% of males echolocate at (e.g. sMh95)
	#Number of males that have a peak frequency lower than the value of the peak frequency threshold under which 95% of males echolocate at (e.g. sMh95)
	#Proportion of males (e.g. pMp95) calculated from the above mentioned number of males (e.g. nM95) and females (e.g. nF95)
#MeaRMSE: ABC estimated minus true value of the proportion of males (used for RMSE calculation)
#pMp95RMSE, pMp99RMSE, pMp999RMSE: exclusion method estimate (95, 99 and 99.9%CI respectively) of the proportion of males minus true value (used for RMSE calculation)
#Pacc: pacc parameter of the ABC algorithm 
#Alpha_Len: Alpha parameter of the ABC algorithm
#Nbsim: Initial number of simulations = nb_simul

#TRUEpM: true proportion of males (only applicable when running simulations)
#Ntot: number of individual points in the dataset
#Nmal: number of male data points in the dataset
#Nfem: number of female data points in the dataset
	#vectors used for labels
		Errors<-c("MeaRMSE","pMp95RMSE","pMp99RMSE","pMp999RMSE")
		Parmas<-c("Pacc","Alpha_Len","Nbsim")
		Suma<-c("pMmea","pMmed","pMhl","pMhh","fMmea","fMmed","fMhl","fMhh","fFmea","fFmed","fFhl","fFhh","SDmea","SDmed","SDhl","SDhh",
		"sFl95","nF95","sMh95","nM95","pMp95"	,"sFl99","nF99","sMh99","nM99","pMp99",	"sFl999","nF999","sMh999","nM999","pMp999",
		Errors,Parmas,	"TRUEpM","Ntot","Nmal","Nfem")

		Reps<-c(paste("Rep",1:Nrep,sep=""))
	#Number of Columns of the summaries
		Ncol<-length(Suma)
	#Array
		ResA<-array(0,dim=c(Nrep,Ncol,dim(mordec)[1]),dimnames=list(Reps,Suma,NULL))

#  ###########################################################################################  #
#  ###########################################################################################  #
#  ###########################################################################################  #
              #### Part III				RUNS THE ABC and EXCLUSION METHODS				     	  ####
#  ###########################################################################################  #
#  ###########################################################################################  #
#  ###########################################################################################  #
		
StartTime<-Sys.time()

#First start of the calculations loop [here for different model parameters as defined in 'mordec']
	for (ds in 1: dim(mordec)[1]) {

#For each replicate
		for (i in 1:Nrep) {


#### 1.	ABC method ####
#Simulations of echolocation datasets with known proportion of males and females
	PRealSim(Nmal=mordec[ds,"Nmale"],Nfem=mordec[ds,"Nfemale"],Mfreq=MaleFreq,Ffreq=FemaleFreq,SdFreq=SdFreqFM) 

if(exists("Rh_ABC_rej") == TRUE) remove(Rh_ABC_rej)
	Rh_ABC_rej<-ABC_sequential(method="Lenormand", model=EchoSim,prior=Rh_prior, nb_simul=nb_simul, summary_stat_target=sum_stat_obs,
		p_acc_min=mordec[ds,"Pac"],alpha = mordec[ds,"Alpha"],prior_test= Rh_Constraints)

#Defines the Credibility interval for the estimate fo the parameters
	HDIval<-0.95
#Proportion of males (mean / median / lower and upper bounds of the Highest density interval of the estimation)
	ResA[i,"pMmea",ds]<-mean(Rh_ABC_rej$param[,1])
	ResA[i,"pMmed",ds]<-median(Rh_ABC_rej$param[,1])
	ResA[i,c("pMhl","pMhh"),ds]<-range(hdi(Rh_ABC_rej$param[,1], credMass=HDIval))
#Peak frequency of males (mean / median / lower and upper bounds of the Highest density interval of the estimation)
	ResA[i,"fMmea",ds]<-mean(Rh_ABC_rej$param[,2])
	ResA[i,"fMmed",ds]<-median(Rh_ABC_rej$param[,2])
	ResA[i,c("fMhl","fMhh"),ds]<-range(hdi(Rh_ABC_rej$param[,2], credMass=HDIval))
#Peak frequency of females (mean / median / lower and upper bounds of the Highest density interval of the estimation)
	ResA[i,"fFmea",ds]<-mean(Rh_ABC_rej$param[,3])
	ResA[i,"fFmed",ds]<-median(Rh_ABC_rej$param[,3])
	ResA[i,c("fFhl","fFhh"),ds]<-range(hdi(Rh_ABC_rej$param[,3], credMass=HDIval))
#SD of peak frequency within each sex (same for both sexes)(mean / median / lower and upper bounds of the Highest density interval of the estimation)
	ResA[i,"SDmea",ds]<-mean(Rh_ABC_rej$param[,4])
	ResA[i,"SDmed",ds]<-median(Rh_ABC_rej$param[,4])
	ResA[i,c("SDhl","SDhh"),ds]<-range(hdi(Rh_ABC_rej$param[,4], credMass=HDIval))


#### 2. Practical method ####

#Gets the different CI of the distribution (for males and females separately) recovered by the ABC method
	MCI<-qnorm(p=c(0.95,0.99,0.999), mean = ResA[i,"fMmea",ds], sd =ResA[i,"SDmea",ds], lower.tail = TRUE, log.p = FALSE)
	FCI<-qnorm(p=c(0.05,0.01,0.001), mean = ResA[i,"fFmea",ds], sd =ResA[i,"SDmea",ds], lower.tail = TRUE, log.p = FALSE)

#Number of males and females kept after removing the overlap zone
	ResA[i,"nM95",ds]<-length(which(dataE$mean < min(MCI[1],FCI[1]))==TRUE)
	ResA[i,"nF95",ds]<-length(which(dataE$mean > max(MCI[1],FCI[1]))==TRUE)
	ResA[i,"nM99",ds]<-length(which(dataE$mean < min(MCI[2],FCI[2]))==TRUE)
	ResA[i,"nF99",ds]<-length(which(dataE$mean > max(MCI[2],FCI[2]))==TRUE)
	ResA[i,"nM999",ds]<-length(which(dataE$mean < min(MCI[3],FCI[3]))==TRUE)
	ResA[i,"nF999",ds]<-length(which(dataE$mean > max(MCI[3],FCI[3]))==TRUE)

ResA[i,c("sFl95","sFl99","sFl999"),ds]<-FCI
ResA[i,c("sMh95","sMh99","sMh999"),ds]<-MCI

ResA[i,"pMp95",ds]<-ResA[i,"nM95",ds]/(ResA[i,"nM95",ds]+ResA[i,"nF95",ds])
ResA[i,"pMp99",ds]<-ResA[i,"nM99",ds]/(ResA[i,"nM99",ds]+ResA[i,"nF99",ds])
ResA[i,"pMp999",ds]<-ResA[i,"nM999",ds]/(ResA[i,"nM999",ds]+ResA[i,"nF999",ds])
	}

#	ResA[,,ds]<-as.matrix(Res)
#Showing report with some indication of time remaining (take this as a rought estimate)
	RunTimeHour<-difftime(Sys.time(),StartTime,units="hours" )
	RunTimeMin<-difftime(Sys.time(),StartTime,units="min" )
	RunTimePerSetHour<-RunTimeHour / ds
	RunTimePerSetMin<-RunTimeMin / ds
	cat("TIMING",ds,"/",dim(mordec)[1]," parameters combinations, Remaining ",round((dim(mordec)[1] - ds) *RunTimePerSetHour,1)," h; ",round((dim(mordec)[1] - ds) *RunTimePerSetMin,1)," min; ","Time per set (h/min)",RunTimePerSetHour,RunTimePerSetMin, "\n")
	flush.console()
}
EndTime<-Sys.time() ; EndTime - StartTime 
##END OF THE calculation LOOP

#  ###########################################################################################  #
#  ###########################################################################################  #
#  ###########################################################################################  #
                    #### 					POST ESTIMATION PROCESSING						  ####
#  ###########################################################################################  #
#  ###########################################################################################  #
#  ###########################################################################################  #

#Adds the parameters used in the simulation
	ResA[,"Nbsim",]<-nb_simul

for (i in 1:dim(mordec)[1])	{
	ResA[,"TRUEpM",i]<-mordec[i,"Pmale"]
	ResA[,"Ntot",i]<-mordec[i,"Ntot"]
	ResA[,"Nmal",i]<-mordec[i,"Nmale"]
	ResA[,"Nfem",i]<-mordec[i,"Nfemale"]
	ResA[,"Alpha_Len",i]<-mordec[i,"Alpha"]
	ResA[,"Pacc",i]<-mordec[i,"Pac"]
	}

#Calculates RMSE
	ResA[,"MeaRMSE",]<-ResA[,"pMmea",]-ResA[,"TRUEpM",]
	ResA[,"pMp95RMSE",]<-ResA[,"pMp95",]-ResA[,"TRUEpM",]
	ResA[,"pMp99RMSE",]<-ResA[,"pMp99",]-ResA[,"TRUEpM",]
	ResA[,"pMp999RMSE",]<-ResA[,"pMp999",]-ResA[,"TRUEpM",]

#RMSE for each parameters combination
	RMSE <-function(X1) {sqrt(mean((X1)^2))}
	SumA<-cbind(mordec,t(apply(ResA[,c(Errors),],c(2,3),RMSE)))

#Exports the results as .csv file (with possibility to re-import and reform the array)
	Epi<-ResA
	EpiC<-melt(Epi, varnames=c("X","Y","Z"))
	dir.create(file.path(Run), showWarnings = FALSE)
	write.csv(EpiC,paste(Run,"/ABC_",format(Sys.time(),"%Y-%m-%d"),"_Results.csv",sep=""))

#Possibility to re-import the results from another Run
#EpiC<-read.csv("D:/SEB/R/R-3.4.0patched/adata/Run-Test1/ABC_2018-01-30_Results.csv",h=TRUE)
#Epiq<-acast(EpiC,  X~ Z ~ Y)

#  ###########################################################################################  #
#  ###########################################################################################  #
#  ###########################################################################################  #
                    ####						PLOTTING OF RESULTS						  ####
#  ###########################################################################################  #
#  ###########################################################################################  #
#  ###########################################################################################  #
	

#### PROPORTION OF MALES ###

pdf(paste(Run,"/ABC_",format(Sys.time(),"%Y-%m-%d"),"_",Run,"_Plots.pdf",sep=""),width=30/2.54,height=20/2.54)

par(mfrow=c(2,2),mar=c(4.1, 4.1, 6.1 ,2.1))

	Epiq<-acast(EpiC,  X~ Z ~ Y)
#ABC results MEAN (with different number of recordings explicitly shown)
	boxplot(Epiq[,,"pMmea"] ~ Epiq[,,"TRUEpM"]+ Epiq[,,"Ntot"],xaxt="n",main="ABC Mean",xlab="Simulated",ylab="Estimated",ylim=c(min(Pmale),max(Pmale,Epiq[,,"pMmea"])))
	axis(1,at=1:length(Pmale),labels=Pmale) ; axis(3,at=1:length(mordec$Ntot),labels=mordec$Ntot)
#PRACTICAL METHOD results 95%CI
	boxplot(Epiq[,,"pMp95"] ~ Epiq[,,"TRUEpM"]+ Epiq[,,"Ntot"],xaxt="n",main="Practical method IC95%",xlab="Simulated",ylab="Estimated",ylim=c(min(Pmale),max(Pmale,Epiq[,,"pMp95"])))
	axis(1,at=1:length(Pmale),labels=Pmale) ; axis(3,at=1:length(mordec$Ntot),labels=mordec$Ntot)
#PRACTICAL METHOD results 99%CI
	boxplot(Epiq[,,"pMp99"] ~ Epiq[,,"TRUEpM"]+ Epiq[,,"Ntot"],xaxt="n",main="Practical method IC99%",xlab="Simulated",ylab="Estimated",ylim=c(min(Pmale),max(Pmale,Epiq[,,"pMp99"])))
	axis(1,at=1:length(Pmale),labels=Pmale) ; axis(3,at=1:length(mordec$Ntot),labels=mordec$Ntot)
#PRACTICAL METHOD results 99.9%CI
	boxplot(Epiq[,,"pMp999"] ~ Epiq[,,"TRUEpM"]+ Epiq[,,"Ntot"],xaxt="n",main="Practical method IC99.9%",xlab="Simulated",ylab="Estimated",ylim=c(min(Pmale),max(Pmale,Epiq[,,"pMp999"])))
	axis(1,at=1:length(Pmale),labels=Pmale) ; axis(3,at=1:length(mordec$Ntot),labels=mordec$Ntot)

par(mfrow=c(2,2),mar=c(5.1, 4.1, 4.1 ,2.1))
#ABC results MEAN (with all number of recordings 'collapsed' together)
	boxplot(Epiq[,,"pMmea"] ~ Epiq[,,"TRUEpM"],xaxt="n",main="ABC Mean",xlab="Simulated",ylab="Simulated")
	axis(1,at=1:length(Pmale),labels=Pmale)
#PRACTICAL METHOD results 95%CI
	boxplot(Epiq[,,"pMp95"] ~ Epiq[,,"TRUEpM"],xaxt="n",main="Practical method IC95%",xlab="Simulated",ylab="Estimated")
	axis(1,at=1:length(Pmale),labels=Pmale)
#PRACTICAL METHOD results 99%CI
	boxplot(Epiq[,,"pMp99"] ~ Epiq[,,"TRUEpM"],xaxt="n",main="Practical method IC99%",xlab="Simulated",ylab="Estimated")
	axis(1,at=1:length(Pmale),labels=Pmale)
#PRACTICAL METHOD results 99.9%CI
	boxplot(Epiq[,,"pMp999"] ~ Epiq[,,"TRUEpM"],xaxt="n",main="Practical method IC99.9%",xlab="Simulated",ylab="Estimated")
	axis(1,at=1:length(Pmale),labels=Pmale)

#### RMSE ####

par(mfrow=c(1,1))

#Plots the RMSE of the ABC estimate of the proportion of males (POM)
	plot(SumA[,"MeaRMSE",],type="l",lwd=2,ylim=c(0,0.1),ylab="RMSE",xaxt="n",xlab="POM_Sample size")


#Adds to the plot the RMSE of the three exclusion method estimates (95, 99 and 99.9%CI respectively) of the proportion of males (POM)  
	for (un in Errors) {
		Numb<-which(Errors %in% un)
		points(SumA[,un,],type="o",col=Numb,lwd=2,pch=16)
		}
#Adds the legend (depending on the settings, please, update the position of the legend [here, set at X=5 and Y=0.1])
	legend(5,0.1,legend=Errors,col=1:4,lwd=1,cex=0.6)
#Adds the X-axis labels
	axis(1,at=1:dim(SumA)[1],labels=c(paste(SumA$Pmale,SumA$Ntot,sep="_")),las=1,cex.axis=0.6,las=2)



##### PEAK FREQUENCY ####

par(mfrow=c(1,3))
#ABC results Peak Frequency of MALES
	boxplot(Epiq[,,"fMmea"] ~ Epiq[,,"TRUEpM"]+ Epiq[,,"Ntot"],xaxt="n",main="Males peak freq ",xlab="POM_Sample size",ylab="Estimated")
#Adds the simulated value and the X-axis
	abline(h=c(MaleFreq),lty=2)
	axis(1,at=1:dim(SumA)[1],labels=c(paste(SumA$Pmale,SumA$Ntot,sep="_")),las=1,cex.axis=0.6,las=2)

#ABC results Peak Frequency of FEMALES
	boxplot(Epiq[,,"fFmea"] ~ Epiq[,,"TRUEpM"]+ Epiq[,,"Ntot"],xaxt="n",main="Females peak freq",xlab="POM_Sample size",ylab="Estimated")
#Adds the simulated value and the X-axis
	abline(h=c(FemaleFreq),lty=2) 
	axis(1,at=1:dim(SumA)[1],labels=c(paste(SumA$Pmale,SumA$Ntot,sep="_")),las=1,cex.axis=0.6,las=2)

#ABC results Peak Frequency of FEMALES
	boxplot(Epiq[,,"SDmea"] ~ Epiq[,,"TRUEpM"]+ Epiq[,,"Ntot"],xaxt="n",main="SD of peak freq",xlab="POM_Sample size",ylab="Estimated")
#Adds the simulated value and the X-axis
	abline(h=c(SdFreqFM),lty=2) 
	axis(1,at=1:dim(SumA)[1],labels=c(paste(SumA$Pmale,SumA$Ntot,sep="_")),las=1,cex.axis=0.6,las=2)

#Send everything to the PDF file
	dev.off()

