#ABC-acoustic version 1.0
#S2 File: R-script for ABC analysis with empirical data
#Script 2
#For updates/upgrades of this script, please visit S.J. Puechmaille's Website; currently at http://batlab.ucd.ie/~spuechmaille/
#For a user manual of this script, please, visit S.J. Puechmaille's Website; currently at http://batlab.ucd.ie/~spuechmaille/
#Written by S.J. Puechmaille, 2017
#This script is to test the ABC and exclusion approaches based on empirical datasets (datasets to be provided by the user in a .csv file)
#This script was developped as part of the following work; Please, cite this work if you use this script.
#Lehnen L, Schorcht W, Karst I, Biedermann M, Kerth G, Puechmaille SJ (submitted) Using Approximate Bayesian Computation to infer sex ratios from acoustic data. PLoS ONE.

#Built and tested under
#R version 3.4.0 Patched (2017-05-25 r72742)
#Platform: x86_64-w64-mingw32/x64 (64-bit)
#Running under: Windows 10 x64 (build 16299)

#install.packages("EasyABC")		#tested with EasyABC_1.5
#install.packages("HDInterval")	#tested with HDInterval_0.1.3
#install.packages("reshape2")		#tested with reshape2_1.4.2 
#install.packages("plyr")		#tested with plyr_1.8.4 
	library("EasyABC")
	library(reshape2)
	library("HDInterval")
	library(plyr)

#  ###########################################################################################  #
#  ###########################################################################################  #
#  ###########################################################################################  #
          ####Part I		Loads and prepares the data from EMPIRICAL dataset			       ####
#  ###########################################################################################  #
#  ###########################################################################################  #
#  ###########################################################################################  #
	
#Sets the working directory
	setwd("D:/SEB/R/R-3.4.0patched/adata/Lisa/")
	rm(list=ls(all=TRUE))

#All data used in Lehnen et al. (2018, PLoS ONE)
#This file contains data in 3 columns (with the headers as presented below) !!!!! [without these exact headers, the script will not work] !!!!!
#Column 1: named 'Fmean': peak frequency (or any acoustic parameter of interest)
#Column 2: named 'Colo': name of the Colony (or site or any other level of interest)
#Column 3: named 'Filename': File name (indicating the grouping or the calls)
#Enter below the name of the input .csv file formatted as described above (for the data from Lehnen et al. (2018, PLoS ONE), 'S1_Table.csv')
	InputCSVfile<-"S1_Table.csv"
	AlldataIN<-read.csv(InputCSVfile)
	head(AlldataIN) ; dim(AlldataIN)

#Colony of interest (as per the information appearing in the 'Colo' column)
	Colony<-"Thu47"

#Threshold of peak frequency under which the calls are removed from the dataset [in kHz]
	PF.Thresh<-100

#Automatically selects the data from the colony of interest
	Keep<-which(AlldataIN$Colo == Colony)
	dataIN<-AlldataIN[Keep,]

#Data from each call must be entered
#Here, select if you want your data to be averaged over 'Recordings' or if you want to carry out the analysis at the 'Calls' level 
	Level<-"Recordings"
		#OR
	#Level<-"Calls"

#Extra info to use in the file name
	extra<-"_Example"

#Automatically generates a Run 'name' from the info provided above(used if files/results are exported)
	Run<-paste("Run_",Colony,"_",Level,extra,sep="")

if(Level == "Recordings") {
	#If multiple calls are present per individual/recording, takes the average after removing calls below a certain peak frequency determined by the 'PF.Thresh' argument
		Res1<-dataIN[,c("Filename","Fmean")]
	#Removes individual calls below a certain peak frequency determined by the 'PF.Thresh' argument
		OUTs<-which(Res1$Fmean< PF.Thresh)
		if(length(OUTs)>0) Res2<-Res1[-OUTs,]
		if(length(OUTs)== 0) Res2<-Res1
		dataX1<-ddply(Res2, .(Res2$Filename),function(df) mean(df[,"Fmean"]))
		dataXsd<-ddply(Res2, .(Res2$Filename),function(df) sd(df[,"Fmean"]))
		dataXNcall<-ddply(Res2, .(Res2$Filename),function(df) length(df[,"Fmean"]))
		dataX<-cbind(dataX1,dataXsd$V1,dataXNcall$V1) ; colnames(dataX)<-c("Filename","mean","sd","Ncalls")
	range(dataX$mean);mean(dataX$mean)
	data<-dataX
	}

if(Level == "Calls") {
	#Removes calls below a certain PF
		OUT<-which(dataIN$Fmean<PF.Thresh)
		if(length(OUT)>0) data<-dataIN[-OUT,c("Filename","Fmean")]
		if(length(OUT)== 0) data<-dataIN[,c("Filename","Fmean")]
	colnames(data)<-c("Filename","mean")
	range(data$mean);mean(data$mean);dim(data)[1]
	}

#dim(data) ; head(data) 

#To filer out Recordings with high standard deviation , set the 'FilterSD' parameter to 'TRUE' , otherwise, to 'FALSE'
FilterSD<-"FALSE"

if(FilterSD == TRUE) {
	#Threshold of sd above which the recordings are removed from the dataset
		SD.Thresh<-1
#Can filter out Recordings with high standard deviation (suggesting the presence of calls from multiple individuals)
	data<-data[-which(data$sd > SD.Thresh),]
	range(data$mean);mean(data$mean);dim(data)[1]
	}

#Creates a function to simulate mixes of males and females
	EchoSim<-function(x) {
	#Number of calls/Recordings depending on what was chosen above
		Nindiv<-dim(dataE)[1]
	#Proportion of males calls/Recordings
		PROPmal<-x[1]
	#Average peak frequency of males
		AVmal<-x[2]
	#Average peak frequency of females
		AVfem<-x[3]
	#Sd of peak frequency (same for males and females)
		SdFreq<-x[4]

	#Gets the number of males and females calls/Recordings 
		NmalSIM<-round(Nindiv*PROPmal,0)
		NfemSIM<-Nindiv-NmalSIM

	#Creates their peak frequency distribution
		MAL<<-rnorm(NmalSIM,AVmal,SdFreq)
		FEM<<-rnorm(NfemSIM,AVfem,SdFreq)
			dataSIM<<-as.data.frame(matrix(0,(NmalSIM+NfemSIM),2,dimnames=list(NULL,c("mean","sex"))))
			dataSIM$mean<<-c(MAL,FEM)
			dataSIM$sex<<-c(rep("M",NmalSIM),rep("F",NfemSIM))
	#Calculates the Kolmogorov-smirnov distance between the simulated and observed dataset
		ksD<-ks.test(c(MAL,FEM),c(dataE$mean),exact=FALSE) [1]	
		ksD1<-as.numeric(ksD[1])
	#Outputs the summary statistics
		c(median(c(MAL,FEM)),mean(c(MAL,FEM)),sd(c(MAL,FEM)),ksD1)
	}


#  ###########################################################################################  #
#  ###########################################################################################  #
#  ###########################################################################################  #
        ##### Part II		Runs THE ABC and Exclusion methods on EMPIRICAL dataset			####
#  ###########################################################################################  #
#  ###########################################################################################  #
#  ###########################################################################################  #
	
###STEP 1
StartTime<-Sys.time()


##### START of the parameters for the model ####

#Number of repetition (the ABC analyses can be repeated multiple times if needed)
	Nrep<-1

#ABC parameters
	nb_simul=1000
	alpha_Lenormand<-0.4		#seq(from=0.1,to=0.4,by=0.1)
	pacc<-0.01				#c(0.001,0.005,seq(from=0.01,to=0.05,by=0.01))
	mordec<- expand.grid(Alpha= alpha_Lenormand,Pac=pacc)
#	set.seed(1) 

##### END of parameters for the model ###



#### STEP 2 ####

#List with priors for the ABC algorithm
#Below, 4 priors (type of distribution, mimimum , maximum) are provided; in order:
	#1-proportion of males
	#2-average peak frequency of males
	#3-average peak frequency of females
	#4-sd of peak frequency for males (also here sd of peak frequency for females)
		Rh_prior=list(c("unif",0,1),c("unif",104,107),c("unif",108,111),c("unif",0.8,1.2))

#Constraints between model parameters. Each parameter is designated with "X1", "X2", ... in the same order as in the prior definition.
	Rh_Constraints<-"X2 < X3 & X3-X2 < 5 & X3-X2 > 2.5"


#### STEP 3 ####

#Large array to place all the results
#Number of Columns to store the summaries
	Errors<-c("MeaRMSE","pMp95RMSE","pMp99RMSE","pMp999RMSE")
	Parmas<-c("Pacc","Alpha_Len","Nbsim")
	Suma<-c("pMmea","pMmed","pMhl","pMhh",	"fMmea","fMmed","fMhl","fMhh",	"fFmea","fFmed","fFhl","fFhh",	"SDmea","SDmed","SDhl","SDhh",
	"sFl95","nF95","sMh95","nM95","pMp95",	"sFl99","nF99","sMh99","nM99","pMp99",	"sFl999","nF999","sMh999","nM999","pMp999",
	Errors,Parmas,	"TRUEpM","Ntot","Nmal","Nfem")
	Ncol<-length(Suma)
	ResA<-array(0,dim=c(Nrep,Ncol,dim(mordec)[1]),dimnames=list(NULL,Suma,NULL))


#### STEP 4 ####

#Runs the ABC estimation of sex-ratio
	dataE<-data

#Summary statistics from the observed dataset
	#1.Median peak frequency (males and females together)
	#2.Mean peak frequency (males and females together)
	#3.Sd of peak frequency (males and females together)
	#4.Kolmogorov-smirnov distance between the observed and observed dataset [0]
		sum_stat_obs<-c(median(dataE$mean),mean(dataE$mean),sd(dataE$mean),0) ; sum_stat_obs

#Start of the calculations loop
	for (ds in 1: dim(mordec)[1]) {

#Object to place the summary stats of the results
	Res<-as.data.frame(matrix(0,Nrep,(Ncol)))
	colnames(Res)<-Suma

#For each replicate
for (i in 1:Nrep) {

#ABC method

if(exists("Rh_ABC_rej") == TRUE) remove(Rh_ABC_rej)
	Rh_ABC_rej<-ABC_sequential(method="Lenormand", model=EchoSim,prior=Rh_prior, nb_simul=nb_simul, summary_stat_target=sum_stat_obs,
		p_acc_min=mordec[ds,"Pac"],alpha = mordec[ds,"Alpha"],prior_test= Rh_Constraints)

#Defines the Credibility interval for the estimate fo the parameters
	HDIval<-0.95
#Proportion of males
	Res[i,"pMmea"]<-mean(Rh_ABC_rej$param[,1])
	Res[i,"pMmed"]<-median(Rh_ABC_rej$param[,1])
	Res[i,c("pMhl","pMhh")]<-range(hdi(Rh_ABC_rej$param[,1], credMass=HDIval))
#Peak frequency of males
	Res[i,"fMmea"]<-mean(Rh_ABC_rej$param[,2])
	Res[i,"fMmed"]<-median(Rh_ABC_rej$param[,2])
	Res[i,c("fMhl","fMhh")]<-range(hdi(Rh_ABC_rej$param[,2], credMass=HDIval))
#Peak frequency of females
	Res[i,"fFmea"]<-mean(Rh_ABC_rej$param[,3])
	Res[i,"fFmed"]<-median(Rh_ABC_rej$param[,3])
	Res[i,c("fFhl","fFhh")]<-range(hdi(Rh_ABC_rej$param[,3], credMass=HDIval))
#SD of peak frequency within each sex (same for both sexes)
	Res[i,"SDmea"]<-mean(Rh_ABC_rej$param[,4])
	Res[i,"SDmed"]<-median(Rh_ABC_rej$param[,4])
	Res[i,c("SDhl","SDhh")]<-range(hdi(Rh_ABC_rej$param[,4], credMass=HDIval))


#### Practical method ###

#Gets the CI of the (here normal) distribution (for males and females separately) recovered by the ABC method
	MCI<-qnorm(p=c(0.95,0.99,0.999), mean = Res[i,"fMmea"], sd =Res[i,"SDmea"], lower.tail = TRUE, log.p = FALSE)
	FCI<-qnorm(p=c(0.05,0.01,0.001), mean = Res[i,"fFmea"], sd =Res[i,"SDmea"], lower.tail = TRUE, log.p = FALSE)

#Number of males and females kept after removing the overlap zone
	Res[i,"nM95"]<-length(which(dataE$mean < min(MCI[1],FCI[1]))==TRUE)
	Res[i,"nF95"]<-length(which(dataE$mean > max(MCI[1],FCI[1]))==TRUE)
	Res[i,"nM99"]<-length(which(dataE$mean < min(MCI[2],FCI[2]))==TRUE)
	Res[i,"nF99"]<-length(which(dataE$mean > max(MCI[2],FCI[2]))==TRUE)
	Res[i,"nM999"]<-length(which(dataE$mean < min(MCI[3],FCI[3]))==TRUE)
	Res[i,"nF999"]<-length(which(dataE$mean > max(MCI[3],FCI[3]))==TRUE)
#Fills in the limits of the exclusion method
	Res[i,c("sFl95","sFl99","sFl999")]<-FCI
	Res[i,c("sMh95","sMh99","sMh999")]<-MCI
#Fills in the estimates of the exclusion method
	Res[i,"pMp95"]<-Res[i,"nM95"]/(Res[i,"nM95"]+Res[i,"nF95"])
	Res[i,"pMp99"]<-Res[i,"nM99"]/(Res[i,"nM99"]+Res[i,"nF99"])
	Res[i,"pMp999"]<-Res[i,"nM999"]/(Res[i,"nM999"]+Res[i,"nF999"])
	}

	ResA[,,ds]<-as.matrix(Res)
#Showing report
	RunTimeHour<-difftime(Sys.time(),StartTime,units="hours" )
	RunTimeMin<-difftime(Sys.time(),StartTime,units="min" )
	RunTimePerSetHour<-RunTimeHour / ds
	RunTimePerSetMin<-RunTimeMin / ds
	cat("TIMING",ds,"/",dim(mordec)[1],", Remaining ",round((dim(mordec)[1] - ds) *RunTimePerSetHour,1)," h; ",round((dim(mordec)[1] - ds) *RunTimePerSetMin,1)," min; ","Time per set (h/min)",RunTimePerSetHour,RunTimePerSetMin, "\n")
	flush.console()
}
EndTime<-Sys.time() ; EndTime - StartTime 
##END OF THE calculation LOOP

#Fills in how many recordings are available plus some parameters of the ABC approach
	ResA[,"Ntot",]<-dim(dataE)[1]
	ResA[,"Nbsim",]<-nb_simul
	ResA[,"Alpha_Len",]<-alpha_Lenormand
	ResA[,"Pacc",]<-pacc

#Shows the ABC estimate and its 95% credibility interval (and the number of data points in the observed dataset)
	ResA[,c("pMmea","pMhl","pMhh","Ntot"),]

#  ###########################################################################################  #
#  ###########################################################################################  #
#  ###########################################################################################  #
  #### Part III		PLOTS the results of the ABC and Exclusion methods on EMPIRICAL dataset	  ####
#  ###########################################################################################  #
#  ###########################################################################################  #
#  ###########################################################################################  #
	
# ################## #
  ####	PLOTS		####
# ################## #

#Exports the results as .csv file (with possibility to later and re-import and reform the array [not implemented here])
	Epi<-ResA
	EpiC<-melt(Epi, varnames=c("X","Y","Z"))
	dir.create(file.path(Run), showWarnings = FALSE)
	write.csv(EpiC,paste(Run,"/ABC_Results.csv",sep=""))

pdf(paste(Run,"/ABC_output_",Run,".pdf",sep=""),width=30/2.54,height=20/2.54)

ABC_Res<-Rh_ABC_rej
par(mfrow=c(1,4))
	plot(density(ABC_Res$stats[,1]),main="Summary stat 1") ; abline(v=sum_stat_obs[1],lty=2)
	plot(density(ABC_Res$stats[,2]),main="Summary stat 2") ; abline(v=sum_stat_obs[2],lty=2)
	plot(density(ABC_Res$stats[,3]),main="Summary stat 3") ; abline(v=sum_stat_obs[3],lty=2)
	plot(density(ABC_Res$stats[,4]),main="Summary stat 4") ; abline(v=sum_stat_obs[4],lty=2)

par(mfrow=c(1,3))
plot(density(ABC_Res$param[,1],from=as.numeric(Rh_prior[[1]][2]),to=as.numeric(Rh_prior[[1]][3])),main=paste("Prop. of males=",round(median(ABC_Res$param[,1]),2),sep=""),yaxt="n",ylab="")
	abline(v=c(as.numeric(Rh_prior[[1]][2]),as.numeric(Rh_prior[[1]][3])),lty=2)
plot(density(ABC_Res$param[,2],from=as.numeric(Rh_prior[[2]][2]),to=as.numeric(Rh_prior[[3]][3])),main=paste("PF males/females=",round(median(ABC_Res$param[,2]),1),"/",round(median(ABC_Res$param[,3]),1),sep=""),yaxt="n",ylab="")
		abline(v=c(as.numeric(Rh_prior[[2]][2]),as.numeric(Rh_prior[[2]][3])),lty=2,col="black")
	points(density(ABC_Res$param[,3],from=as.numeric(Rh_prior[[2]][2]),to=as.numeric(Rh_prior[[3]][3])),main="Mean female Freq",type="l",col="red")
		abline(v=c(as.numeric(Rh_prior[[3]][2]),as.numeric(Rh_prior[[3]][3])),lty=2,col="red")
plot(density(ABC_Res$param[,4],from=as.numeric(Rh_prior[[4]][2]),to=as.numeric(Rh_prior[[4]][3])),main=paste("SD peak Freq=",round(median(ABC_Res$param[,4]),2),sep=""),yaxt="n",ylab="")
		abline(v=c(as.numeric(Rh_prior[[4]][2]),as.numeric(Rh_prior[[4]][3])),lty=2,col="black")
dev.off()

